/**
   The MySensors Arduino library handles the wireless radio link and protocol
   between your home built sensors/actuators and HA controller of choice.
   The sensors forms a self healing radio network with optional repeaters. Each
   repeater and gateway builds a routing tables in EEPROM which keeps track of the
   network topology allowing messages to be routed to nodes.

   Created by Henrik Ekblad <henrik.ekblad@mysensors.org>
   Copyright (C) 2013-2015 Sensnology AB
   Full contributor list: https://github.com/mysensors/Arduino/graphs/contributors

   Documentation: http://www.mysensors.org
   Support Forum: http://forum.mysensors.org

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   version 2 as published by the Free Software Foundation.

 *******************************

   REVISION HISTORY
   Version 1.0 - Henrik EKblad

   DESCRIPTION
   This sketch provides an example how to implement a humidity/temperature
   sensor using DHT11/DHT-22
   http://www.mysensors.org/build/humidity
*/

#include <SPI.h>
#include <MySensor.h>
#include <DHT.h>
#include <Bounce2.h>

#define CHILD_ID 3 //Vera electric range device info
#define PHOTOCELL_PIN  2  // Photocell (LDR) reading input
#define PUSHBUTTON_PIN  4 // Time select for timer (cooking time)
#define BUZZ_PIN 5 //Buzzer pin output

#define CHILD_ID_HUM 0
#define CHILD_ID_TEMP 1
#define HUMIDITY_SENSOR_DIGITAL_PIN 3

MySensor gw;

Bounce debouncer = Bounce();
//boolean timesup=false;

byte ledpins[5] = {A0, A1, A2, A3, A4};
unsigned long SLEEP_TIME = 30 * 1000; // Sleep time between DHT reads (in milliseconds)

byte photocell_OldValue = -1; //Photocell old state

unsigned long cook_lastMillis = 0;
unsigned long COOK_MINUTE = 60 * 1000; //1 min multiplier
//unsigned long COOK_MINUTE = 10 * 1000; //10 sec multiplier for demo/debug
unsigned int cook_times[5] = {60, 120, 180, 240, 300}; //times for timer in minutes
//unsigned int cook_times[5] = {1, 2, 3, 4, 5}; //times for timer in minutes for demo/debug
byte cook_timeindex = 0;
boolean cook_toggle = false; //Start of buzztimer

byte sound_state = LOW;
unsigned long sound_lastMillis;
unsigned long sound_ontime = 500;
unsigned long sound_offtime = 500;

DHT dht;	//Used AM2320, has both i2c and onewire protocol, used onewire
float lastTemp, lastHum;
boolean metric = true;
unsigned long dht_lastMillis = 0; //Start time for waittoupdate function

MyMessage msg(CHILD_ID, V_LIGHT); //Vera device for electric range state
MyMessage msgHum(CHILD_ID_HUM, V_HUM); //Vera device for Humidity
MyMessage msgTemp(CHILD_ID_TEMP, V_TEMP); //Vera device for Temperature

void setup()
{
  gw.begin();


  for (int x = 0; x < 5; x++) {
    pinMode(ledpins[x], OUTPUT);
    digitalWrite(ledpins[x], LOW);
  }

  pinMode(PHOTOCELL_PIN, INPUT_PULLUP); // Setup photocell input

  pinMode(BUZZ_PIN, OUTPUT); // Setup photocell input
  digitalWrite(BUZZ_PIN, LOW);

  pinMode(PUSHBUTTON_PIN, INPUT_PULLUP); // Setup photocell input

  debouncer.attach(PUSHBUTTON_PIN);
  debouncer.interval(150);

  dht.setup(HUMIDITY_SENSOR_DIGITAL_PIN);

  // Send the Sketch Version Information to the Gateway
  gw.sendSketchInfo("Electric Range Monitor", "1.0");

  // Register all sensors to gw (they will be created as child devices)
  gw.present(CHILD_ID, S_LIGHT);
  gw.present(CHILD_ID_HUM, S_HUM);
  gw.present(CHILD_ID_TEMP, S_TEMP);

  metric = gw.getConfig().isMetric;
}

void loop()
{

  processphotocell(); //process for getting photocell value

  processdht(); //Process for AM2320 sensor

  processbuzz(); //Process interval timer and buzzer

}


void processbuzz() {

  debouncer.update();

  if (cook_toggle == true) {

    unsigned long cook_currentMillis = millis();

    if (debouncer.fell()) {         // check button pressed
      cook_timeindex++;
      if (cook_timeindex >= 5)
        cook_timeindex = 0;
      cook_lastMillis = millis();
      digitalWrite(BUZZ_PIN, LOW);

      //Serial.println(cook_timeindex); //debug
    }

    ledarray(LOW);
    digitalWrite(ledpins[cook_timeindex], HIGH);
    //Serial.println((cook_currentMillis - cook_lastMillis)/1000); //debug

    //buzzer beeps 500ms interval
    if ((cook_currentMillis - cook_lastMillis) > (cook_times[cook_timeindex] * COOK_MINUTE)) {
      unsigned long sound_currentMillis = millis();
      if ((sound_state == HIGH) && (sound_currentMillis - sound_lastMillis >= sound_ontime))
      {
        sound_state = LOW;  // Turn it off
        sound_lastMillis = sound_currentMillis;  // Remember the time
        digitalWrite(BUZZ_PIN, sound_state);  // Deactivate sound
      }
      else if ((sound_state == LOW) && (sound_currentMillis - sound_lastMillis >= sound_offtime))
      {
        sound_state = HIGH;  // Turn it off
        sound_lastMillis = sound_currentMillis;  // Remember the time
        digitalWrite(BUZZ_PIN, sound_state);  // Activate sound
      }

    }
  }
}


void processdht() {
  unsigned long dht_currentMillis = millis();

  if ((dht_currentMillis - dht_lastMillis) > SLEEP_TIME ) { //sleep a bit

    dht_lastMillis = dht_currentMillis;

    delay(dht.getMinimumSamplingPeriod());

    float temperature = dht.getTemperature();
    if (isnan(temperature)) {
      Serial.println("Failed reading temperature from DHT");
    } else if (temperature != lastTemp) {
      lastTemp = temperature;
      if (!metric) {
        temperature = dht.toFahrenheit(temperature);
      }
      gw.send(msgTemp.set(temperature, 1));
      Serial.print("T: ");
      Serial.println(temperature);
    }

    float humidity = dht.getHumidity();
    if (isnan(humidity)) {
      Serial.println("Failed reading humidity from DHT");
    } else if (humidity != lastHum) {
      lastHum = humidity;
      gw.send(msgHum.set(humidity, 1));
      Serial.print("H: ");
      Serial.println(humidity);
    }
  }
}

//Checks for photocell input staus change
void processphotocell() {
  int value = digitalRead(PHOTOCELL_PIN);
  if (value != photocell_OldValue) {

    // Send in the new value
    gw.send(msg.set(value == HIGH ? 1 : 0));

    if (value == HIGH && cook_toggle == false) {
      cook_lastMillis = millis();
      cook_toggle = true;
    } else if (value == LOW) {
      cook_toggle = false;
      digitalWrite(BUZZ_PIN, LOW);
      cook_timeindex = 0;
      ledarray(LOW);
    }
    photocell_OldValue = value;
  }
}

void ledarray(byte state) {
  for (int x = 0; x < 5; x++) {
    digitalWrite(ledpins[x], state);
  }
}


